@@ -3,9 +3,6 @@ use std::hash::Hasher;
 
 use libafl::inputs::{HasTargetBytes, Input};
 
-#[cfg(any(target_os = "linux", target_os = "android"))]
-use libafl::bolts::os::find_mapping_for_path;
-
 use libafl_targets::drcov::{DrCovBasicBlock, DrCovWriter};
 
 #[cfg(target_arch = "aarch64")]
@@ -19,7 +16,6 @@ use capstone::{
     Capstone, Insn,
 };
 
-use core::cell::RefCell;
 #[cfg(target_arch = "x86_64")]
 use frida_gum::instruction_writer::X86Register;
 #[cfg(target_arch = "aarch64")]
@@ -27,24 +23,41 @@ use frida_gum::instruction_writer::{Aarch64Register, IndexMode};
 use frida_gum::{
     instruction_writer::InstructionWriter,
     stalker::{StalkerOutput, Transformer},
-    CpuContext,
+    CpuContext, ModuleDetails, ModuleMap,
 };
 use frida_gum::{Gum, Module, PageProtection};
 #[cfg(target_arch = "aarch64")]
 use num_traits::cast::FromPrimitive;
 
 use rangemap::RangeMap;
-use std::{path::PathBuf, rc::Rc};
+
+use nix::sys::mman::{mmap, MapFlags, ProtFlags};
 
 use crate::{asan_rt::AsanRuntime, FridaOptions};
 
+#[cfg(feature = "cmplog")]
+use crate::cmplog_rt::CmpLogRuntime;
+
+#[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
+enum CmplogOperandType {
+    Regid(capstone::RegId),
+    Imm(u64),
+    Cimm(u64),
+    Mem(capstone::RegId, capstone::RegId, i32, u32),
+}
+
+#[cfg(any(target_os = "macos", target_os = "ios"))]
+const ANONYMOUS_FLAG: MapFlags = MapFlags::MAP_ANON;
+#[cfg(not(any(target_os = "macos", target_os = "ios")))]
+const ANONYMOUS_FLAG: MapFlags = MapFlags::MAP_ANONYMOUS;
+
 /// An helper that feeds [`FridaInProcessExecutor`] with user-supplied instrumentation
 pub trait FridaHelper<'a> {
     /// Access to the stalker `Transformer`
     fn transformer(&self) -> &Transformer<'a>;
 
     /// Register a new thread with this `FridaHelper`
-    fn register_thread(&self);
+    fn register_thread(&mut self);
 
     /// Called prior to execution of an input
     fn pre_exec<I: Input + HasTargetBytes>(&mut self, input: &I);
@@ -57,6 +70,8 @@ pub trait FridaHelper<'a> {
 
     /// pointer to the frida coverage map
     fn map_ptr(&mut self) -> *mut u8;
+
+    fn ranges(&self) -> &RangeMap<usize, (u16, String)>;
 }
 
 /// (Default) map size for frida coverage reporting
@@ -73,8 +88,11 @@ pub struct FridaInstrumentationHelper<'a> {
     transformer: Option<Transformer<'a>>,
     #[cfg(target_arch = "aarch64")]
     capstone: Capstone,
-    asan_runtime: Rc<RefCell<AsanRuntime>>,
-    ranges: RangeMap<usize, (u16, &'a str)>,
+    asan_runtime: AsanRuntime,
+    #[cfg(feature = "cmplog")]
+    cmplog_runtime: CmpLogRuntime,
+    ranges: RangeMap<usize, (u16, String)>,
+    module_map: ModuleMap,
     options: &'a FridaOptions,
     drcov_basic_blocks: Vec<DrCovBasicBlock>,
 }
@@ -85,8 +103,8 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
     }
 
     /// Register the current thread with the [`FridaInstrumentationHelper`]
-    fn register_thread(&self) {
-        self.asan_runtime.borrow().register_thread();
+    fn register_thread(&mut self) {
+        self.asan_runtime.register_thread();
     }
 
     fn pre_exec<I: Input + HasTargetBytes>(&mut self, input: &I) {
@@ -94,7 +112,6 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
         let slice = target_bytes.as_slice();
         //println!("target_bytes: {:02x?}", slice);
         self.asan_runtime
-            .borrow()
             .unpoison(slice.as_ptr() as usize, slice.len());
     }
 
@@ -109,9 +126,9 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
 
         if self.options.asan_enabled() {
             if self.options.asan_detect_leaks() {
-                self.asan_runtime.borrow_mut().check_for_leaks();
+                self.asan_runtime.check_for_leaks();
             }
-            self.asan_runtime.borrow_mut().reset_allocations();
+            self.asan_runtime.reset_allocations();
         }
     }
 
@@ -122,6 +139,10 @@ impl<'a> FridaHelper<'a> for FridaInstrumentationHelper<'a> {
     fn map_ptr(&mut self) -> *mut u8 {
         self.map.as_mut_ptr()
     }
+
+    fn ranges(&self) -> &RangeMap<usize, (u16, String)> {
+        &self.ranges
+    }
 }
 
 /// Helper function to get the size of a module's CODE section from frida
@@ -208,14 +229,38 @@ fn pc(context: &CpuContext) -> usize {
 /// The implementation of the [`FridaInstrumentationHelper`]
 impl<'a> FridaInstrumentationHelper<'a> {
     /// Constructor function to create a new [`FridaInstrumentationHelper`], given a `module_name`.
-    #[allow(clippy::clippy::too_many_lines)]
+    #[allow(clippy::too_many_lines)]
     #[must_use]
     pub fn new(
         gum: &'a Gum,
         options: &'a FridaOptions,
         _harness_module_name: &str,
-        modules_to_instrument: &'a [PathBuf],
+        modules_to_instrument: &'a [&str],
     ) -> Self {
+        // workaround frida's frida-gum-allocate-near bug:
+        unsafe {
+            for _ in 0..512 {
+                mmap(
+                    std::ptr::null_mut(),
+                    128 * 1024,
+                    ProtFlags::PROT_NONE,
+                    ANONYMOUS_FLAG | MapFlags::MAP_PRIVATE | MapFlags::MAP_NORESERVE,
+                    -1,
+                    0,
+                )
+                .expect("Failed to map dummy regions for frida workaround");
+                mmap(
+                    std::ptr::null_mut(),
+                    4 * 1024 * 1024,
+                    ProtFlags::PROT_NONE,
+                    ANONYMOUS_FLAG | MapFlags::MAP_PRIVATE | MapFlags::MAP_NORESERVE,
+                    -1,
+                    0,
+                )
+                .expect("Failed to map dummy regions for frida workaround");
+            }
+        }
+
         let mut helper = Self {
             map: [0u8; MAP_SIZE],
             previous_pc: [0u64; 1],
@@ -231,32 +276,26 @@ impl<'a> FridaInstrumentationHelper<'a> {
                 .build()
                 .expect("Failed to create Capstone object"),
             asan_runtime: AsanRuntime::new(options.clone()),
+            #[cfg(feature = "cmplog")]
+            cmplog_runtime: CmpLogRuntime::new(),
             ranges: RangeMap::new(),
+            module_map: ModuleMap::new_from_names(modules_to_instrument),
             options,
             drcov_basic_blocks: vec![],
         };
 
         if helper.options().stalker_enabled() {
-            for (id, module_name) in modules_to_instrument.iter().enumerate() {
-                let (lib_start, lib_end) = find_mapping_for_path(module_name.to_str().unwrap());
-                println!(
-                    "including range {:x}-{:x} for {:?}",
-                    lib_start, lib_end, module_name
-                );
-                helper.ranges.insert(
-                    lib_start..lib_end,
-                    (id as u16, module_name.to_str().unwrap()),
-                );
+            for (i, module) in helper.module_map.values().iter().enumerate() {
+                let range = module.range();
+                let start = range.base_address().0 as usize;
+                helper
+                    .ranges
+                    .insert(start..(start + range.size()), (i as u16, module.path()));
             }
-
             if let Some(suppressed_specifiers) = helper.options().dont_instrument_locations() {
                 for (module_name, offset) in suppressed_specifiers {
-                    let (lib_start, _) = find_mapping_for_path(
-                        std::fs::canonicalize(&module_name)
-                            .unwrap()
-                            .to_str()
-                            .unwrap(),
-                    );
+                    let module_details = ModuleDetails::with_name(module_name).unwrap();
+                    let lib_start = module_details.range().base_address().0 as usize;
                     println!("removing address: {:#x}", lib_start + offset);
                     helper
                         .ranges
@@ -274,6 +313,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
                 for instruction in basic_block {
                     let instr = instruction.instr();
                     let address = instr.address();
+                    //println!("block @ {:x} transformed to {:x}", address, output.writer().pc());
                     //println!("address: {:x} contains: {:?}", address, helper.ranges.contains_key(&(address as usize)));
                     if helper.ranges.contains_key(&(address as usize)) {
                         if first {
@@ -284,20 +324,14 @@ impl<'a> FridaInstrumentationHelper<'a> {
                             }
                             if helper.options().drcov_enabled() {
                                 instruction.put_callout(|context| {
-                                    let real_address = match helper
-                                        .asan_runtime
-                                        .borrow()
-                                        .real_address_for_stalked(pc(&context))
-                                    {
-                                        Some(address) => *address,
-                                        None => pc(&context),
-                                    };
+                                    let real_address =
+                                        helper.asan_runtime.real_address_for_stalked(pc(&context));
                                     //let (range, (id, name)) = helper.ranges.get_key_value(&real_address).unwrap();
                                     //println!("{}:0x{:016x}", name, real_address - range.start);
                                     helper
                                         .drcov_basic_blocks
                                         .push(DrCovBasicBlock::new(real_address, real_address + 4));
-                                })
+                                });
                             }
                         }
 
@@ -306,7 +340,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
                             todo!("Implement ASAN for non-aarch64 targets");
                             #[cfg(target_arch = "aarch64")]
                             if let Ok((basereg, indexreg, displacement, width, shift, extender)) =
-                                helper.is_interesting_instruction(address, instr)
+                                helper.asan_is_interesting_instruction(address, instr)
                             {
                                 helper.emit_shadow_check(
                                     address,
@@ -320,19 +354,36 @@ impl<'a> FridaInstrumentationHelper<'a> {
                                 );
                             }
                         }
+                        if helper.options().cmplog_enabled() {
+                            #[cfg(not(target_arch = "aarch64"))]
+                            todo!("Implement cmplog for non-aarch64 targets");
+                            #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
+                            // check if this instruction is a compare instruction and if so save the registers values
+                            if let Ok((op1, op2)) =
+                                helper.cmplog_is_interesting_instruction(address, instr)
+                            {
+                                //emit code that saves the relevant data in runtime(passes it to x0, x1)
+                                helper.emit_comparison_handling(address, &output, op1, op2);
+                            }
+                        }
+
                         if helper.options().asan_enabled() || helper.options().drcov_enabled() {
-                            helper.asan_runtime.borrow_mut().add_stalked_address(
+                            helper.asan_runtime.add_stalked_address(
                                 output.writer().pc() as usize - 4,
                                 address as usize,
                             );
                         }
                     }
-                    instruction.keep()
+                    instruction.keep();
                 }
             });
             helper.transformer = Some(transformer);
             if helper.options().asan_enabled() || helper.options().drcov_enabled() {
-                helper.asan_runtime.borrow_mut().init(modules_to_instrument);
+                helper.asan_runtime.init(gum, modules_to_instrument);
+            }
+            #[cfg(feature = "cmplog")]
+            if helper.options.cmplog_enabled() {
+                helper.cmplog_runtime.init();
             }
         }
         helper
@@ -340,7 +391,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
 
     #[inline]
     fn options(&self) -> &FridaOptions {
-        &self.options
+        self.options
     }
     #[cfg(target_arch = "aarch64")]
     #[inline]
@@ -349,6 +400,312 @@ impl<'a> FridaInstrumentationHelper<'a> {
         Aarch64Register::from_u32(regint as u32).unwrap()
     }
 
+    #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
+    #[inline]
+    /// Emit the instrumentation code which is responsible for opernads value extraction and cmplog map population
+    fn emit_comparison_handling(
+        &self,
+        _address: u64,
+        output: &StalkerOutput,
+        op1: CmplogOperandType,
+        op2: CmplogOperandType,
+    ) {
+        let writer = output.writer();
+
+        // Preserve x0, x1:
+        writer.put_stp_reg_reg_reg_offset(
+            Aarch64Register::X0,
+            Aarch64Register::X1,
+            Aarch64Register::Sp,
+            -(16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i32) as i64,
+            IndexMode::PreAdjust,
+        );
+
+        // make sure operand1 value is saved into x0
+        match op1 {
+            CmplogOperandType::Imm(value) | CmplogOperandType::Cimm(value) => {
+                writer.put_ldr_reg_u64(Aarch64Register::X0, value);
+            }
+            CmplogOperandType::Regid(reg) => {
+                let reg = self.writer_register(reg);
+                match reg {
+                    Aarch64Register::X0 | Aarch64Register::W0 => {}
+                    Aarch64Register::X1 | Aarch64Register::W1 => {
+                        writer.put_mov_reg_reg(Aarch64Register::X0, Aarch64Register::X1);
+                    }
+                    _ => {
+                        if !writer.put_mov_reg_reg(Aarch64Register::X0, reg) {
+                            writer.put_mov_reg_reg(Aarch64Register::W0, reg);
+                        }
+                    }
+                }
+            }
+            CmplogOperandType::Mem(basereg, indexreg, displacement, _width) => {
+                let basereg = self.writer_register(basereg);
+                let indexreg = if indexreg.0 != 0 {
+                    Some(self.writer_register(indexreg))
+                } else {
+                    None
+                };
+
+                // calculate base+index+displacment into x0
+                let displacement = displacement
+                    + if basereg == Aarch64Register::Sp {
+                        16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i32
+                    } else {
+                        0
+                    };
+
+                if indexreg.is_some() {
+                    if let Some(indexreg) = indexreg {
+                        writer.put_add_reg_reg_reg(Aarch64Register::X0, basereg, indexreg);
+                    }
+                } else {
+                    match basereg {
+                        Aarch64Register::X0 | Aarch64Register::W0 => {}
+                        Aarch64Register::X1 | Aarch64Register::W1 => {
+                            writer.put_mov_reg_reg(Aarch64Register::X0, Aarch64Register::X1);
+                        }
+                        _ => {
+                            if !writer.put_mov_reg_reg(Aarch64Register::X0, basereg) {
+                                writer.put_mov_reg_reg(Aarch64Register::W0, basereg);
+                            }
+                        }
+                    }
+                }
+
+                //add displacement
+                writer.put_add_reg_reg_imm(
+                    Aarch64Register::X0,
+                    Aarch64Register::X0,
+                    displacement as u64,
+                );
+
+                //deref into x0 to get the real value
+                writer.put_ldr_reg_reg_offset(Aarch64Register::X0, Aarch64Register::X0, 0u64);
+            }
+        }
+
+        // make sure operand2 value is saved into x1
+        match op2 {
+            CmplogOperandType::Imm(value) | CmplogOperandType::Cimm(value) => {
+                writer.put_ldr_reg_u64(Aarch64Register::X1, value);
+            }
+            CmplogOperandType::Regid(reg) => {
+                let reg = self.writer_register(reg);
+                match reg {
+                    Aarch64Register::X1 | Aarch64Register::W1 => {}
+                    Aarch64Register::X0 | Aarch64Register::W0 => {
+                        writer.put_ldr_reg_reg_offset(
+                            Aarch64Register::X1,
+                            Aarch64Register::Sp,
+                            0u64,
+                        );
+                    }
+                    _ => {
+                        if !writer.put_mov_reg_reg(Aarch64Register::X1, reg) {
+                            writer.put_mov_reg_reg(Aarch64Register::W1, reg);
+                        }
+                    }
+                }
+            }
+            CmplogOperandType::Mem(basereg, indexreg, displacement, _width) => {
+                let basereg = self.writer_register(basereg);
+                let indexreg = if indexreg.0 != 0 {
+                    Some(self.writer_register(indexreg))
+                } else {
+                    None
+                };
+
+                // calculate base+index+displacment into x1
+                let displacement = displacement
+                    + if basereg == Aarch64Register::Sp {
+                        16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i32
+                    } else {
+                        0
+                    };
+
+                if indexreg.is_some() {
+                    if let Some(indexreg) = indexreg {
+                        match indexreg {
+                            Aarch64Register::X0 | Aarch64Register::W0 => {
+                                match basereg {
+                                    Aarch64Register::X1 | Aarch64Register::W1 => {
+                                        // x0 is overwrittern indexreg by op1 value.
+                                        // x1 is basereg
+
+                                        // Preserve x2, x3:
+                                        writer.put_stp_reg_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::X3,
+                                            Aarch64Register::Sp,
+                                            -(16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i32) as i64,
+                                            IndexMode::PreAdjust,
+                                        );
+
+                                        //reload indexreg to x2
+                                        writer.put_ldr_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::Sp,
+                                            0u64,
+                                        );
+                                        //add them into basereg==x1
+                                        writer.put_add_reg_reg_reg(
+                                            basereg,
+                                            basereg,
+                                            Aarch64Register::X2,
+                                        );
+
+                                        // Restore x2, x3
+                                        assert!(writer.put_ldp_reg_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::X3,
+                                            Aarch64Register::Sp,
+                                            16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i64,
+                                            IndexMode::PostAdjust,
+                                        ));
+                                    }
+                                    _ => {
+                                        // x0 is overwrittern indexreg by op1 value.
+                                        // basereg is not x1 nor x0
+
+                                        //reload indexreg to x1
+                                        writer.put_ldr_reg_reg_offset(
+                                            Aarch64Register::X1,
+                                            Aarch64Register::Sp,
+                                            0u64,
+                                        );
+                                        //add basereg into indexreg==x1
+                                        writer.put_add_reg_reg_reg(
+                                            Aarch64Register::X1,
+                                            basereg,
+                                            Aarch64Register::X1,
+                                        );
+                                    }
+                                }
+                            }
+                            Aarch64Register::X1 | Aarch64Register::W1 => {
+                                match basereg {
+                                    Aarch64Register::X0 | Aarch64Register::W0 => {
+                                        // x0 is overwrittern basereg by op1 value.
+                                        // x1 is indexreg
+
+                                        // Preserve x2, x3:
+                                        writer.put_stp_reg_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::X3,
+                                            Aarch64Register::Sp,
+                                            -(16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i32) as i64,
+                                            IndexMode::PreAdjust,
+                                        );
+
+                                        //reload basereg to x2
+                                        writer.put_ldr_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::Sp,
+                                            0u64,
+                                        );
+                                        //add basereg into indexreg==x1
+                                        writer.put_add_reg_reg_reg(
+                                            indexreg,
+                                            Aarch64Register::X2,
+                                            indexreg,
+                                        );
+
+                                        // Restore x2, x3
+                                        assert!(writer.put_ldp_reg_reg_reg_offset(
+                                            Aarch64Register::X2,
+                                            Aarch64Register::X3,
+                                            Aarch64Register::Sp,
+                                            16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i64,
+                                            IndexMode::PostAdjust,
+                                        ));
+                                    }
+                                    _ => {
+                                        // indexreg is x1
+                                        // basereg is not x0 and not x1
+
+                                        //add them into x1
+                                        writer.put_add_reg_reg_reg(indexreg, basereg, indexreg);
+                                    }
+                                }
+                            }
+                            _ => {
+                                match basereg {
+                                    Aarch64Register::X0 | Aarch64Register::W0 => {
+                                        //basereg is overwrittern by op1 value
+                                        //index reg is not x0 nor x1
+
+                                        //reload basereg to x1
+                                        writer.put_ldr_reg_reg_offset(
+                                            Aarch64Register::X1,
+                                            Aarch64Register::Sp,
+                                            0u64,
+                                        );
+                                        //add indexreg to basereg==x1
+                                        writer.put_add_reg_reg_reg(
+                                            Aarch64Register::X1,
+                                            Aarch64Register::X1,
+                                            indexreg,
+                                        );
+                                    }
+                                    _ => {
+                                        //basereg is not x0, can be x1
+                                        //index reg is not x0 nor x1
+
+                                        //add them into x1
+                                        writer.put_add_reg_reg_reg(
+                                            Aarch64Register::X1,
+                                            basereg,
+                                            indexreg,
+                                        );
+                                    }
+                                }
+                            }
+                        }
+                    }
+                } else {
+                    match basereg {
+                        Aarch64Register::X1 | Aarch64Register::W1 => {}
+                        Aarch64Register::X0 | Aarch64Register::W0 => {
+                            // x0 is overwrittern basereg by op1 value.
+                            //reload basereg to x1
+                            writer.put_ldr_reg_reg_offset(
+                                Aarch64Register::X1,
+                                Aarch64Register::Sp,
+                                0u64,
+                            );
+                        }
+                        _ => {
+                            writer.put_mov_reg_reg(Aarch64Register::W1, basereg);
+                        }
+                    }
+                }
+
+                // add displacement
+                writer.put_add_reg_reg_imm(
+                    Aarch64Register::X1,
+                    Aarch64Register::X1,
+                    displacement as u64,
+                );
+                //deref into x1 to get the real value
+                writer.put_ldr_reg_reg_offset(Aarch64Register::X1, Aarch64Register::X1, 0u64);
+            }
+        }
+
+        //call cmplog runtime to populate the values map
+        writer.put_bytes(&self.cmplog_runtime.ops_save_register_and_blr_to_populate());
+
+        // Restore x0, x1
+        assert!(writer.put_ldp_reg_reg_reg_offset(
+            Aarch64Register::X0,
+            Aarch64Register::X1,
+            Aarch64Register::Sp,
+            16 + frida_gum_sys::GUM_RED_ZONE_SIZE as i64,
+            IndexMode::PostAdjust,
+        ));
+    }
+
     #[cfg(target_arch = "aarch64")]
     #[inline]
     fn emit_shadow_check(
@@ -384,7 +741,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
             writer.put_b_label(after_report_impl);
 
             self.current_report_impl = writer.pc();
-            writer.put_bytes(self.asan_runtime.borrow().blob_report());
+            writer.put_bytes(self.asan_runtime.blob_report());
 
             writer.put_label(after_report_impl);
         }
@@ -521,18 +878,18 @@ impl<'a> FridaInstrumentationHelper<'a> {
         }
         // Insert the check_shadow_mem code blob
         match width {
-            1 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_byte()),
-            2 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_halfword()),
-            3 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_3bytes()),
-            4 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_dword()),
-            6 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_6bytes()),
-            8 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_qword()),
-            12 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_12bytes()),
-            16 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_16bytes()),
-            24 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_24bytes()),
-            32 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_32bytes()),
-            48 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_48bytes()),
-            64 => writer.put_bytes(&self.asan_runtime.borrow().blob_check_mem_64bytes()),
+            1 => writer.put_bytes(&self.asan_runtime.blob_check_mem_byte()),
+            2 => writer.put_bytes(&self.asan_runtime.blob_check_mem_halfword()),
+            3 => writer.put_bytes(&self.asan_runtime.blob_check_mem_3bytes()),
+            4 => writer.put_bytes(&self.asan_runtime.blob_check_mem_dword()),
+            6 => writer.put_bytes(&self.asan_runtime.blob_check_mem_6bytes()),
+            8 => writer.put_bytes(&self.asan_runtime.blob_check_mem_qword()),
+            12 => writer.put_bytes(&self.asan_runtime.blob_check_mem_12bytes()),
+            16 => writer.put_bytes(&self.asan_runtime.blob_check_mem_16bytes()),
+            24 => writer.put_bytes(&self.asan_runtime.blob_check_mem_24bytes()),
+            32 => writer.put_bytes(&self.asan_runtime.blob_check_mem_32bytes()),
+            48 => writer.put_bytes(&self.asan_runtime.blob_check_mem_48bytes()),
+            64 => writer.put_bytes(&self.asan_runtime.blob_check_mem_64bytes()),
             _ => false,
         };
 
@@ -626,7 +983,7 @@ impl<'a> FridaInstrumentationHelper<'a> {
 
     #[cfg(target_arch = "aarch64")]
     #[inline]
-    fn is_interesting_instruction(
+    fn asan_is_interesting_instruction(
         &self,
         _address: u64,
         instr: &Insn,
@@ -676,6 +1033,70 @@ impl<'a> FridaInstrumentationHelper<'a> {
         Err(())
     }
 
+    #[cfg(all(feature = "cmplog", target_arch = "aarch64"))]
+    #[inline]
+    /// Check if the current instruction is cmplog relevant one(any opcode which sets the flags)
+    fn cmplog_is_interesting_instruction(
+        &self,
+        _address: u64,
+        instr: &Insn,
+    ) -> Result<(CmplogOperandType, CmplogOperandType), ()> {
+        // We only care for compare instrunctions - aka instructions which set the flags
+        match instr.mnemonic().unwrap() {
+            "cmp" | "ands" | "subs" | "adds" | "negs" | "ngcs" | "sbcs" | "bics" | "cls" => (),
+            _ => return Err(()),
+        }
+        let operands = self
+            .capstone
+            .insn_detail(instr)
+            .unwrap()
+            .arch_detail()
+            .operands();
+        if operands.len() != 2 {
+            return Err(());
+        }
+
+        let operand1 = if let Arm64Operand(arm64operand) = operands.first().unwrap() {
+            match arm64operand.op_type {
+                Arm64OperandType::Reg(regid) => Some(CmplogOperandType::Regid(regid)),
+                Arm64OperandType::Imm(val) => Some(CmplogOperandType::Imm(val as u64)),
+                Arm64OperandType::Mem(opmem) => Some(CmplogOperandType::Mem(
+                    opmem.base(),
+                    opmem.index(),
+                    opmem.disp(),
+                    self.instruction_width(instr, &operands),
+                )),
+                Arm64OperandType::Cimm(val) => Some(CmplogOperandType::Cimm(val as u64)),
+                _ => return Err(()),
+            }
+        } else {
+            None
+        };
+
+        let operand2 = if let Arm64Operand(arm64operand2) = operands.last().unwrap() {
+            match arm64operand2.op_type {
+                Arm64OperandType::Reg(regid) => Some(CmplogOperandType::Regid(regid)),
+                Arm64OperandType::Imm(val) => Some(CmplogOperandType::Imm(val as u64)),
+                Arm64OperandType::Mem(opmem) => Some(CmplogOperandType::Mem(
+                    opmem.base(),
+                    opmem.index(),
+                    opmem.disp(),
+                    self.instruction_width(instr, &operands),
+                )),
+                Arm64OperandType::Cimm(val) => Some(CmplogOperandType::Cimm(val as u64)),
+                _ => return Err(()),
+            }
+        } else {
+            None
+        };
+
+        if operand1.is_some() && operand2.is_some() {
+            Ok((operand1.unwrap(), operand2.unwrap()))
+        } else {
+            Err(())
+        }
+    }
+
     #[inline]
     fn emit_coverage_mapping(&mut self, address: u64, output: &StalkerOutput) {
         let writer = output.writer();
@@ -704,7 +1125,6 @@ impl<'a> FridaInstrumentationHelper<'a> {
         }
         #[cfg(target_arch = "x86_64")]
         {
-            println!("here");
             writer.put_lea_reg_reg_offset(X86Register::Rsp, X86Register::Rsp, -(redzone_size));
             writer.put_push_reg(X86Register::Rdi);
             writer.put_mov_reg_address(
